from python_pachyderm.service import Service


class LicenseMixin:
    def activate_license(self, activation_code, expires=None):
        """
        Activates the license service. Returns a `TokenInfo` object.

        Params:

        * `activation_code`: A string specifying a Pachyderm enterprise
        activation code. New users can obtain trial activation codes.
        * `expires`: An optional `Timestamp` object indicating when this
        activation code will expire. This should not generally be set (it's
        primarily used for testing), and is only applied if it's earlier than
        the signed expiration time in `activation_code`.
        """
        return self._req(
            Service.LICENSE,
            "Activate",
            activation_code=activation_code,
            expires=expires,
        )

    def add_cluster(
        self,
        id,
        address,
        secret=None,
        user_address=None,
        cluster_deployment_id=None,
        enterprise_server=None,
    ):
        """
        Register a cluster with the license service.

        Params:

        * `id`: A unique ID to identify the cluster.
        * `address`: A GRPC address for the license server to reach the cluster.
        * `secret`: Optional. A shared secret for the cluster to use
        to authenticate.
        If not specified, a random secret will be generated and returned.
        """
        return self._req(
            Service.LICENSE,
            "AddCluster",
            id=id,
            address=address,
            secret=secret,
            user_address=user_address,
            cluster_deployment_id=cluster_deployment_id,
            enterprise_server=enterprise_server,
        )

    def update_cluster(
        self, id, address, user_address=None, cluster_deployment_id=None
    ):
        """
        Update a cluster registered with the license service.

        Params:

        * `id`: The unique ID to identify the cluster.
        * `address`: A GRPC address for the license server to reach the cluster.
        """
        self._req(
            Service.LICENSE,
            "UpdateCluster",
            id=id,
            address=address,
            user_address=user_address,
            cluster_deployment_id=cluster_deployment_id,
        )

    def delete_cluster(self, id):
        """
        Delete a cluster registered with the license service.

        Params:

        * `id`: The unique ID to identify the cluster.
        """
        self._req(Service.LICENSE, "DeleteCluster", id=id)

    def list_clusters(self):
        """
        List clusters registered with the license service.
        """
        return self._req(Service.LICENSE, "ListClusters")

    def get_activation_code(self):
        """
        Returns the enterprise code used to activate the server.
        """
        return self._req(Service.LICENSE, "GetActivationCode")

    def delete_all_license(self):
        """
        Remove all clusters and deactivate the license service.
        """
        self._req(Service.LICENSE, "DeleteAll")

    def list_user_clusters(self):
        """
        Lists all clusters available to user.
        """
        return self._req(Service.LICENSE, "ListUserClusters")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from python_pachyderm.proto.v2.license import license_pb2 as python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2


class APIStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Activate = channel.unary_unary(
                '/license_v2.API/Activate',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateResponse.FromString,
                )
        self.GetActivationCode = channel.unary_unary(
                '/license_v2.API/GetActivationCode',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeResponse.FromString,
                )
        self.DeleteAll = channel.unary_unary(
                '/license_v2.API/DeleteAll',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllResponse.FromString,
                )
        self.AddCluster = channel.unary_unary(
                '/license_v2.API/AddCluster',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterResponse.FromString,
                )
        self.DeleteCluster = channel.unary_unary(
                '/license_v2.API/DeleteCluster',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterResponse.FromString,
                )
        self.ListClusters = channel.unary_unary(
                '/license_v2.API/ListClusters',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersResponse.FromString,
                )
        self.UpdateCluster = channel.unary_unary(
                '/license_v2.API/UpdateCluster',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterResponse.FromString,
                )
        self.Heartbeat = channel.unary_unary(
                '/license_v2.API/Heartbeat',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatResponse.FromString,
                )
        self.ListUserClusters = channel.unary_unary(
                '/license_v2.API/ListUserClusters',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersResponse.FromString,
                )


class APIServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Activate(self, request, context):
        """Activate enables the license service by setting the enterprise activation
        code to serve.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetActivationCode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteAll(self, request, context):
        """DeleteAll deactivates the server and removes all data.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddCluster(self, request, context):
        """CRUD operations for the pachds registered with this server.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteCluster(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListClusters(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateCluster(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Heartbeat(self, request, context):
        """Heartbeat is the RPC registered pachds make to the license server
        to communicate their status and fetch updates.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListUserClusters(self, request, context):
        """Lists all clusters available to user
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_APIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Activate': grpc.unary_unary_rpc_method_handler(
                    servicer.Activate,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateResponse.SerializeToString,
            ),
            'GetActivationCode': grpc.unary_unary_rpc_method_handler(
                    servicer.GetActivationCode,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeResponse.SerializeToString,
            ),
            'DeleteAll': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteAll,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllResponse.SerializeToString,
            ),
            'AddCluster': grpc.unary_unary_rpc_method_handler(
                    servicer.AddCluster,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterResponse.SerializeToString,
            ),
            'DeleteCluster': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteCluster,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterResponse.SerializeToString,
            ),
            'ListClusters': grpc.unary_unary_rpc_method_handler(
                    servicer.ListClusters,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersResponse.SerializeToString,
            ),
            'UpdateCluster': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateCluster,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterResponse.SerializeToString,
            ),
            'Heartbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.Heartbeat,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatResponse.SerializeToString,
            ),
            'ListUserClusters': grpc.unary_unary_rpc_method_handler(
                    servicer.ListUserClusters,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'license_v2.API', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class API(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Activate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/Activate',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ActivateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetActivationCode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/GetActivationCode',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.GetActivationCodeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/DeleteAll',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteAllResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddCluster(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/AddCluster',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.AddClusterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteCluster(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/DeleteCluster',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.DeleteClusterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListClusters(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/ListClusters',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListClustersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateCluster(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/UpdateCluster',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.UpdateClusterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Heartbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/Heartbeat',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.HeartbeatResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListUserClusters(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/license_v2.API/ListUserClusters',
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_license_dot_license__pb2.ListUserClustersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from python_pachyderm.proto.v2.auth import auth_pb2 as python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2


class APIStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Activate = channel.unary_unary(
                '/auth_v2.API/Activate',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateResponse.FromString,
                )
        self.Deactivate = channel.unary_unary(
                '/auth_v2.API/Deactivate',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateResponse.FromString,
                )
        self.GetConfiguration = channel.unary_unary(
                '/auth_v2.API/GetConfiguration',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationResponse.FromString,
                )
        self.SetConfiguration = channel.unary_unary(
                '/auth_v2.API/SetConfiguration',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationResponse.FromString,
                )
        self.Authenticate = channel.unary_unary(
                '/auth_v2.API/Authenticate',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateResponse.FromString,
                )
        self.Authorize = channel.unary_unary(
                '/auth_v2.API/Authorize',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeResponse.FromString,
                )
        self.GetPermissions = channel.unary_unary(
                '/auth_v2.API/GetPermissions',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.FromString,
                )
        self.GetPermissionsForPrincipal = channel.unary_unary(
                '/auth_v2.API/GetPermissionsForPrincipal',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsForPrincipalRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.FromString,
                )
        self.WhoAmI = channel.unary_unary(
                '/auth_v2.API/WhoAmI',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIResponse.FromString,
                )
        self.GetRolesForPermission = channel.unary_unary(
                '/auth_v2.API/GetRolesForPermission',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionResponse.FromString,
                )
        self.ModifyRoleBinding = channel.unary_unary(
                '/auth_v2.API/ModifyRoleBinding',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingResponse.FromString,
                )
        self.GetRoleBinding = channel.unary_unary(
                '/auth_v2.API/GetRoleBinding',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingResponse.FromString,
                )
        self.GetOIDCLogin = channel.unary_unary(
                '/auth_v2.API/GetOIDCLogin',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginResponse.FromString,
                )
        self.GetRobotToken = channel.unary_unary(
                '/auth_v2.API/GetRobotToken',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenResponse.FromString,
                )
        self.RevokeAuthToken = channel.unary_unary(
                '/auth_v2.API/RevokeAuthToken',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenResponse.FromString,
                )
        self.RevokeAuthTokensForUser = channel.unary_unary(
                '/auth_v2.API/RevokeAuthTokensForUser',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserResponse.FromString,
                )
        self.SetGroupsForUser = channel.unary_unary(
                '/auth_v2.API/SetGroupsForUser',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserResponse.FromString,
                )
        self.ModifyMembers = channel.unary_unary(
                '/auth_v2.API/ModifyMembers',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersResponse.FromString,
                )
        self.GetGroups = channel.unary_unary(
                '/auth_v2.API/GetGroups',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.FromString,
                )
        self.GetGroupsForPrincipal = channel.unary_unary(
                '/auth_v2.API/GetGroupsForPrincipal',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsForPrincipalRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.FromString,
                )
        self.GetUsers = channel.unary_unary(
                '/auth_v2.API/GetUsers',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersResponse.FromString,
                )
        self.ExtractAuthTokens = channel.unary_unary(
                '/auth_v2.API/ExtractAuthTokens',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensResponse.FromString,
                )
        self.RestoreAuthToken = channel.unary_unary(
                '/auth_v2.API/RestoreAuthToken',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenResponse.FromString,
                )
        self.DeleteExpiredAuthTokens = channel.unary_unary(
                '/auth_v2.API/DeleteExpiredAuthTokens',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensResponse.FromString,
                )
        self.RotateRootToken = channel.unary_unary(
                '/auth_v2.API/RotateRootToken',
                request_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenRequest.SerializeToString,
                response_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenResponse.FromString,
                )


class APIServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Activate(self, request, context):
        """Activate/Deactivate the auth API. 'Activate' sets an initial set of admins
        for the Pachyderm cluster, and 'Deactivate' removes all ACLs, tokens, and
        admins from the Pachyderm cluster, making all data publicly accessable
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Deactivate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetConfiguration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetConfiguration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Authenticate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Authorize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPermissions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPermissionsForPrincipal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WhoAmI(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRolesForPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyRoleBinding(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRoleBinding(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetOIDCLogin(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRobotToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RevokeAuthToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RevokeAuthTokensForUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetGroupsForUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ModifyMembers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetGroups(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetGroupsForPrincipal(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUsers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExtractAuthTokens(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestoreAuthToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteExpiredAuthTokens(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RotateRootToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_APIServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Activate': grpc.unary_unary_rpc_method_handler(
                    servicer.Activate,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateResponse.SerializeToString,
            ),
            'Deactivate': grpc.unary_unary_rpc_method_handler(
                    servicer.Deactivate,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateResponse.SerializeToString,
            ),
            'GetConfiguration': grpc.unary_unary_rpc_method_handler(
                    servicer.GetConfiguration,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationResponse.SerializeToString,
            ),
            'SetConfiguration': grpc.unary_unary_rpc_method_handler(
                    servicer.SetConfiguration,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationResponse.SerializeToString,
            ),
            'Authenticate': grpc.unary_unary_rpc_method_handler(
                    servicer.Authenticate,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateResponse.SerializeToString,
            ),
            'Authorize': grpc.unary_unary_rpc_method_handler(
                    servicer.Authorize,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeResponse.SerializeToString,
            ),
            'GetPermissions': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPermissions,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.SerializeToString,
            ),
            'GetPermissionsForPrincipal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPermissionsForPrincipal,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsForPrincipalRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.SerializeToString,
            ),
            'WhoAmI': grpc.unary_unary_rpc_method_handler(
                    servicer.WhoAmI,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIResponse.SerializeToString,
            ),
            'GetRolesForPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRolesForPermission,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionResponse.SerializeToString,
            ),
            'ModifyRoleBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyRoleBinding,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingResponse.SerializeToString,
            ),
            'GetRoleBinding': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRoleBinding,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingResponse.SerializeToString,
            ),
            'GetOIDCLogin': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOIDCLogin,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginResponse.SerializeToString,
            ),
            'GetRobotToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRobotToken,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenResponse.SerializeToString,
            ),
            'RevokeAuthToken': grpc.unary_unary_rpc_method_handler(
                    servicer.RevokeAuthToken,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenResponse.SerializeToString,
            ),
            'RevokeAuthTokensForUser': grpc.unary_unary_rpc_method_handler(
                    servicer.RevokeAuthTokensForUser,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserResponse.SerializeToString,
            ),
            'SetGroupsForUser': grpc.unary_unary_rpc_method_handler(
                    servicer.SetGroupsForUser,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserResponse.SerializeToString,
            ),
            'ModifyMembers': grpc.unary_unary_rpc_method_handler(
                    servicer.ModifyMembers,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersResponse.SerializeToString,
            ),
            'GetGroups': grpc.unary_unary_rpc_method_handler(
                    servicer.GetGroups,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.SerializeToString,
            ),
            'GetGroupsForPrincipal': grpc.unary_unary_rpc_method_handler(
                    servicer.GetGroupsForPrincipal,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsForPrincipalRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.SerializeToString,
            ),
            'GetUsers': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUsers,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersResponse.SerializeToString,
            ),
            'ExtractAuthTokens': grpc.unary_unary_rpc_method_handler(
                    servicer.ExtractAuthTokens,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensResponse.SerializeToString,
            ),
            'RestoreAuthToken': grpc.unary_unary_rpc_method_handler(
                    servicer.RestoreAuthToken,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenResponse.SerializeToString,
            ),
            'DeleteExpiredAuthTokens': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteExpiredAuthTokens,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensResponse.SerializeToString,
            ),
            'RotateRootToken': grpc.unary_unary_rpc_method_handler(
                    servicer.RotateRootToken,
                    request_deserializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenRequest.FromString,
                    response_serializer=python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'auth_v2.API', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class API(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Activate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/Activate',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ActivateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Deactivate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/Deactivate',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeactivateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetConfiguration(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetConfiguration',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetConfigurationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetConfiguration(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/SetConfiguration',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetConfigurationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Authenticate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/Authenticate',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthenticateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Authorize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/Authorize',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.AuthorizeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPermissions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetPermissions',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPermissionsForPrincipal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetPermissionsForPrincipal',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsForPrincipalRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetPermissionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WhoAmI(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/WhoAmI',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.WhoAmIResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRolesForPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetRolesForPermission',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRolesForPermissionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ModifyRoleBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/ModifyRoleBinding',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyRoleBindingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRoleBinding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetRoleBinding',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRoleBindingResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetOIDCLogin(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetOIDCLogin',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetOIDCLoginResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRobotToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetRobotToken',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetRobotTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RevokeAuthToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/RevokeAuthToken',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RevokeAuthTokensForUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/RevokeAuthTokensForUser',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RevokeAuthTokensForUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetGroupsForUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/SetGroupsForUser',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.SetGroupsForUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ModifyMembers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/ModifyMembers',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ModifyMembersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetGroups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetGroups',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetGroupsForPrincipal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetGroupsForPrincipal',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsForPrincipalRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetGroupsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUsers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/GetUsers',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.GetUsersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExtractAuthTokens(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/ExtractAuthTokens',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.ExtractAuthTokensResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RestoreAuthToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/RestoreAuthToken',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RestoreAuthTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteExpiredAuthTokens(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/DeleteExpiredAuthTokens',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.DeleteExpiredAuthTokensResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RotateRootToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/auth_v2.API/RotateRootToken',
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenRequest.SerializeToString,
            python__pachyderm_dot_proto_dot_v2_dot_auth_dot_auth__pb2.RotateRootTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from python_pachyderm.proto.v2.admin import admin_pb2_grpc as admin_grpc
from python_pachyderm.service import Service


class AdminMixin:
    def inspect_cluster(self):
        """
        Inspects a cluster. Returns a `ClusterInfo` object.
        """
        return self._req(
            Service.ADMIN,
            "InspectCluster",
            req=admin_grpc.google_dot_protobuf_dot_empty__pb2.Empty(),
        )



#!/usr/bin/env python

"""Tests admin-related functionality"""

import python_pachyderm
from python_pachyderm.proto.v2.admin import admin_pb2


def test_inspect_cluster():
    client = python_pachyderm.Client()
    res = client.inspect_cluster()
    assert isinstance(res, admin_pb2.ClusterInfo)



#!/usr/bin/env python

"""Tests of utility functions."""

import os
import json
import tempfile

import python_pachyderm
from python_pachyderm.service import pps_proto
from tests import util

# script that copies a file using just stdlibs
TEST_STDLIB_SOURCE = """
from shutil import copyfile
print("copying")
copyfile("/pfs/{}/file.dat", "/pfs/out/file.dat")
"""

# script that copies a file with padding and colorized output, using
# third-party libraries (defined in `TEST_REQUIREMENTS_SOURCE`.)
TEST_LIB_SOURCE = """
from termcolor import cprint
from leftpad import left_pad

cprint('copying', 'green')

with open('/pfs/{}/file.dat', 'r') as f:
    contents = f.read()

with open('/pfs/out/file.dat', 'w') as f:
    f.write(left_pad(contents, 5))
"""

TEST_REQUIREMENTS_SOURCE = """
# WCGW?
leftpad==0.1.2
termcolor==1.1.0
"""

TEST_PIPELINE_SPEC = """
{
  "pipeline": {
    "name": "foobar"
  },
  "description": "A pipeline that performs image edge detection by using the OpenCV library.",
  "input": {
    "pfs": {
      "glob": "/*",
      "repo": "images"
    }
  },
  "transform": {
    "cmd": [ "python3", "/edges.py" ],
    "image": "pachyderm/opencv"
  }
}
"""


def check_expected_files(client: python_pachyderm.Client, commit, expected):
    for fi in client.walk_file(commit, "/"):
        path = fi.file.path
        assert path in expected, "unexpected path: {}".format(path)
        expected.remove(path)

    for path in expected:
        assert False, "expected path not found: {}".format(path)


def test_put_files():
    client = python_pachyderm.Client()
    client.delete_all()
    repo_name = util.create_test_repo(client, "put_files")

    with tempfile.TemporaryDirectory(suffix="python_pachyderm") as d:
        # create a temporary directory with these files:
        # 0.txt  1.txt  2.txt  3.txt  4.txt  0/0.txt  1/1.txt  2/2.txt
        # 3/3.txt  4/4.txt
        for i in range(5):
            os.makedirs(os.path.join(d, str(i)))

        for j in range(5):
            with open(os.path.join(d, "{}.txt".format(j)), "w") as f:
                f.write(str(j))
            with open(os.path.join(d, str(j), "{}.txt".format(j)), "w") as f:
                f.write(str(j))

        # add the files under both `/` and `/sub` (the latter redundantly to
        # test both for correct path handling and the ability to put files
        # that already exist)
        commit = (repo_name, "master")
        python_pachyderm.put_files(client, d, commit, "/")
        python_pachyderm.put_files(client, d, commit, "/sub")
        python_pachyderm.put_files(client, d, commit, "/sub/")

    expected = set(["/", "/sub/"])
    for i in range(5):
        expected.add("/{}/".format(i))
        expected.add("/{}.txt".format(i))
        expected.add("/{}/{}.txt".format(i, i))
        expected.add("/sub/{}/".format(i))
        expected.add("/sub/{}.txt".format(i))
        expected.add("/sub/{}/{}.txt".format(i, i))

    check_expected_files(client, commit, expected)


def test_put_files_single_file():
    client = python_pachyderm.Client()
    client.delete_all()
    repo_name = util.create_test_repo(client, "put_files_single_file")

    with tempfile.NamedTemporaryFile() as f:
        f.write(b"abcd")
        f.flush()
        commit = (repo_name, "master")
        python_pachyderm.put_files(client, f.name, commit, "/f1.txt")
        python_pachyderm.put_files(client, f.name, commit, "/f/f1")

    expected = set(["/", "/f1.txt", "/f/", "/f/f1"])
    check_expected_files(client, commit, expected)


def test_parse_json_pipeline_spec():
    req = python_pachyderm.parse_json_pipeline_spec(TEST_PIPELINE_SPEC)
    check_pipeline_spec(req)


def test_parse_dict_pipeline_spec():
    req = python_pachyderm.parse_dict_pipeline_spec(json.loads(TEST_PIPELINE_SPEC))
    check_pipeline_spec(req)


def check_pipeline_spec(req):
    assert req == pps_proto.CreatePipelineRequest(
        pipeline=pps_proto.Pipeline(name="foobar"),
        description="A pipeline that performs image edge detection by using the OpenCV library.",
        input=pps_proto.Input(
            pfs=pps_proto.PFSInput(glob="/*", repo="images"),
        ),
        transform=pps_proto.Transform(
            cmd=["python3", "/edges.py"],
            image="pachyderm/opencv",
        ),
    )



import io
import itertools
import tarfile
from contextlib import contextmanager
from typing import Iterable, Iterator, Union, List

from python_pachyderm.pfs import commit_from, Commit, uuid_re
from python_pachyderm.service import pfs_proto, Service
from google.protobuf import empty_pb2, wrappers_pb2


BUFFER_SIZE = 19 * 1024 * 1024


class FileTarstream:
    """
    Implements a file-like interface over a GRPC byte stream,
    so we can use tarfile to decode the file contents.
    """

    def __init__(self, res):
        self.res = res
        self.buf = []

    def __next__(self):
        return next(self.res).value

    def close(self):
        self.res.cancel()

    def read(self, size=-1):
        if self.res.cancelled():
            return b""

        buf = []
        remaining = size if size >= 0 else 2 ** 32

        if self.buf:
            buf.append(self.buf[:remaining])
            self.buf = self.buf[remaining:]
            remaining -= len(buf[-1])

        try:
            while remaining > 0:
                b = next(self)

                if len(b) > remaining:
                    buf.append(b[:remaining])
                    self.buf = b[remaining:]
                else:
                    buf.append(b)

                remaining -= len(buf[-1])
        except StopIteration:
            pass

        return b"".join(buf)


class PFSFile:
    """
    The contents of a file stored in PFS. You can treat these as
    file-like objects, like so:

    ```
    source_file = client.get_file("montage/master", "/montage.png")
    with open("montage.png", "wb") as dest_file:
        shutil.copyfileobj(source_file, dest_file)
    ```
    """

    def __init__(self, stream, is_tar=False):
        if is_tar:
            # Pachyderm's GetFileTar API returns its result (which may include
            # several files, e.g. when getting a directory) as a tar
            # stream--untar the response byte stream as we receive it from
            # GetFileTar.
            # TODO how to handle multiple files in the tar stream?
            f = tarfile.open(fileobj=stream, mode="r|*")
            self._file = f.extractfile(f.next())
        else:
            self._file = stream

    def __iter__(self):
        return self

    def __next__(self):
        x = self.read()
        if not x:
            raise StopIteration
        return x

    def read(self, size=-1):
        return self._file.read(size)

    def close(self):
        self._file.close()


class PFSMixin:
    def create_repo(self, repo_name, description=None, update=None):
        """
        Creates a new `Repo` object in PFS with the given name. Repos are the
        top level data object in PFS and should be used to store data of a
        similar type. For example rather than having a single `Repo` for an
        entire project you might have separate `Repo`s for logs, metrics,
        database dumps etc.

        Params:

        * `repo_name`: Name of the repo.
        * `description`: An optional string describing the repo.
        * `update`: Whether to update if the repo already exists.
        """
        self._req(
            Service.PFS,
            "CreateRepo",
            repo=pfs_proto.Repo(name=repo_name, type="user"),
            description=description,
            update=update,
        )

    def inspect_repo(self, repo_name):
        """
        Returns info about a specific repo. Returns a `RepoInfo` object.

        Params:

        * `repo_name`: Name of the repo.
        """
        return self._req(
            Service.PFS, "InspectRepo", repo=pfs_proto.Repo(name=repo_name, type="user")
        )

    def list_repo(self, type=""):
        """
        Returns info about all repos, as a list of `RepoInfo` objects.

        Params:

        * `type`: the type of (system) repos that should be returned,
        an empty value None or empty string requests all repos.
        """
        return self._req(Service.PFS, "ListRepo", type=type)

    def delete_repo(self, repo_name, force=False):
        """
        Deletes a repo and reclaims the storage space it was using.

        Params:

        * `repo_name`: The name of the repo.
        * `force`: If set to true, the repo will be removed regardless of
          errors. This argument should be used with care.
        """
        self._req(
            Service.PFS,
            "DeleteRepo",
            repo=pfs_proto.Repo(name=repo_name, type="user"),
            force=force,
        )

    def delete_all_repos(self):
        """
        Deletes all repos.
        """
        self._req(Service.PFS, "DeleteAll", req=empty_pb2.Empty())

    def start_commit(self, repo_name, branch, parent=None, description=None) -> Commit:
        """
        Begins the process of committing data to a Repo. Once started you can
        write to the Commit with ModifyFile and when all the data has been
        written you must finish the Commit with FinishCommit. NOTE, data is
        not persisted until FinishCommit is called. A Commit object is
        returned.

        Params:

        * `repo_name`: A string specifying the name of the repo.
        * `branch`: A string specifying the branch name. This is a more
        convenient way to build linear chains of commits. When a commit is
        started with a non-empty branch the value of branch becomes an alias
        for the created Commit. This enables a more intuitive access pattern.
        When the commit is started on a branch the previous head of the branch
        is used as the parent of the commit.
        * `parent`: An optional `Commit` object specifying the parent commit.
        Upon creation the new commit will appear identical to the parent
        commit, data can safely be added to the new commit without affecting
        the contents of the parent commit.
        * `description`: An optional string describing the commit.
        """
        if parent and isinstance(parent, str):
            parent = pfs_proto.Commit(
                id=parent,
                branch=pfs_proto.Branch(
                    repo=pfs_proto.Repo(name=repo_name, type="user"), name=None
                ),
            )
        return self._req(
            Service.PFS,
            "StartCommit",
            parent=parent,
            branch=pfs_proto.Branch(
                repo=pfs_proto.Repo(name=repo_name, type="user"), name=branch
            ),
            description=description,
        )

    def finish_commit(self, commit, description=None, error=None, force=None):
        """
        Ends the process of committing data to a Repo and persists the
        Commit. Once a Commit is finished the data becomes immutable and
        future attempts to write to it with ModifyFile will error.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `description`: An optional string describing this commit.
        * `error`: An optional bool. Used to mark that even though this
        commit is finished, it was interrupted or didn't occur properly.
        * `force`: An optional bool. If true, commit will be forcefully
        finished, even if it breaks provenance.
        """
        self._req(
            Service.PFS,
            "FinishCommit",
            commit=commit_from(commit),
            description=description,
            error=error,
            force=force,
        )

    @contextmanager
    def commit(self, repo_name, branch, parent=None, description=None):
        """
        A context manager for running operations within a commit.

        Params:

        * `repo_name`: A string specifying the name of the repo.
        * `branch`: A string specifying the branch name. This is a more
        convenient way to build linear chains of commits. When a commit is
        started with a non-empty branch the value of branch becomes an alias
        for the created Commit. This enables a more intuitive access pattern.
        When the commit is started on a branch the previous head of the branch
        is used as the parent of the commit.
        * `parent`: An optional `Commit` object specifying the parent commit.
        Upon creation the new commit will appear identical to the parent
        commit, data can safely be added to the new commit without affecting
        the contents of the parent commit.
        * `description`: An optional string describing the commit.
        """
        commit = self.start_commit(repo_name, branch, parent, description)
        try:
            yield commit
        finally:
            self.finish_commit(commit)

    def inspect_commit(
        self,
        commit: Union[str, tuple, dict, Commit, pfs_proto.Commit],
        commit_state: pfs_proto.CommitState = None,
    ):
        """
        Inspects a commit. Yields `CommitInfo` objects.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `commit_state`: An optional int that causes this method to block
        until the commit is in the desired state.
            1: STARTED
            2: READY
            3: FINISHING
            4: FINISHED
        """
        if not isinstance(commit, str):
            return iter(
                [
                    self._req(
                        Service.PFS,
                        "InspectCommit",
                        commit=commit_from(commit),
                        wait=commit_state,
                    )
                ]
            )
        elif uuid_re.match(commit):
            return self._req(
                Service.PFS,
                "InspectCommitSet",
                commit_set=pfs_proto.CommitSet(id=commit),
                wait=commit_state == pfs_proto.CommitState.FINISHED,
            )
        raise ValueError(
            "bad argument: commit should either be a commit ID (str) or a commit-like object"
        )

    def list_commit(
        self,
        repo_name=None,
        to_commit=None,
        from_commit=None,
        number=None,
        reverse=None,
        all=False,
        origin_kind: pfs_proto.OriginKind = 0,
    ):
        """
        Lists commits. Yields `CommitInfo` or `CommitSetInfo` objects.

        Params:

        * `repo_name`: Optional. If only `repo_name` is given, all commits
        in the repo are returned.
        * `to_commit`: Optional. Only the ancestors of `to`, including `to`
        itself, are considered.
        * `from_commit`: Optional. Only the descendants of `from`, including
        `from` itself, are considered.
        * `number`: Optional. Determines how many commits are returned.  If
        `number` is 0, all commits that match the aforementioned criteria are
        returned.
        * `reverse`: Optional. If true, commits are returned oldest to newest.
        * `all`: Optional. If true, all types of commits are returned.
        * `origin_kind`: Optional. Returns only commits of this enum type if
        `repo_name` is provided.
        """
        if repo_name is not None:
            req = pfs_proto.ListCommitRequest(
                repo=pfs_proto.Repo(name=repo_name, type="user"),
                number=number,
                reverse=reverse,
                all=all,
                origin_kind=origin_kind,
            )
            if to_commit is not None:
                req.to.CopyFrom(commit_from(to_commit))
            if from_commit is not None:
                getattr(req, "from").CopyFrom(commit_from(from_commit))
            return self._req(Service.PFS, "ListCommit", req=req)
        else:
            return self._req(Service.PFS, "ListCommitSet")

    def squash_commit(self, commit_id: str):
        """
        Squashes the subcommits of a `Commit` into their children.
        Params:
        * `commit_id`: the id of a `Commit`.
        """
        self._req(
            Service.PFS,
            "SquashCommitSet",
            commit_set=pfs_proto.CommitSet(id=commit_id),
        )

    def drop_commit(self, commit_id: str):
        """
        Drops an entire commit.

        Params:
        * `commit_id`: the id of a `Commit`.
        """
        self._req(
            Service.PFS,
            "DropCommitSet",
            commit_set=pfs_proto.CommitSet(id=commit_id),
        )

    def wait_commit(
        self, commit: Union[str, tuple, dict, Commit, pfs_proto.Commit]
    ) -> List[pfs_proto.CommitInfo]:
        """
        Waits for the specified commit or commit_set to finish and return them.

        Params:
        * `commit`: A `Commit` object, tuple, or str. If passed a commit_set_id,
           then wait for the entire commit_set.
        """
        return list(self.inspect_commit(commit, pfs_proto.CommitState.FINISHED))

    def subscribe_commit(
        self,
        repo_name,
        branch,
        from_commit_id=None,
        state: pfs_proto.CommitState = 1,
        all=False,
        origin_kind: pfs_proto.OriginKind = 0,
    ):
        """
        Yields `CommitInfo` objects as commits occur.

        Params:

        * `repo_name`: A string specifying the name of the repo.
        * `branch`: A string specifying branch to subscribe to.
        * `from_commit_id`: An optional string specifying the commit ID. Only
        commits created since this commit are returned.
        * `state`: The commit state to filter on.
        * `all`: Optional. If true, all types of commits are returned.
        * `origin_kind`: Optional. Returns only commits of this enum type.
        """
        repo = pfs_proto.Repo(name=repo_name, type="user")
        req = pfs_proto.SubscribeCommitRequest(
            repo=repo, branch=branch, state=state, all=all, origin_kind=origin_kind
        )
        if from_commit_id is not None:
            getattr(req, "from").CopyFrom(
                pfs_proto.Commit(repo=repo, id=from_commit_id)
            )
        return self._req(Service.PFS, "SubscribeCommit", req=req)

    def create_branch(
        self,
        repo_name,
        branch_name,
        head_commit=None,
        provenance=None,
        trigger=None,
        new_commit=False,
    ):
        """
        Creates a new branch.

        Params:

        * `repo_name`: A string specifying the name of the repo.
        * `branch_name`: A string specifying the new branch name.
        * `commit`: An optional tuple, string, or `Commit` object representing
           the head commit of the branch.
        * `provenance`: An optional iterable of `Branch` objects representing
          the branch provenance.
        * `trigger`: An optional `Trigger` object controlling when the head of
          `branch_name` is moved.
        * `new_commitset`: A bool, overrides the default behavior of using the
           same Commitset as 'head'
        """
        self._req(
            Service.PFS,
            "CreateBranch",
            branch=pfs_proto.Branch(
                repo=pfs_proto.Repo(name=repo_name, type="user"), name=branch_name
            ),
            head=commit_from(head_commit),
            provenance=provenance,
            trigger=trigger,
            new_commit_set=new_commit,
        )

    def inspect_branch(self, repo_name, branch_name):
        """
        Inspects a branch. Returns a `BranchInfo` object.
        """
        return self._req(
            Service.PFS,
            "InspectBranch",
            branch=pfs_proto.Branch(
                repo=pfs_proto.Repo(name=repo_name, type="user"), name=branch_name
            ),
        )

    def list_branch(self, repo_name, reverse=None):
        """
        Lists the active branch objects on a repo. Returns a list of
        `BranchInfo` objects.

        Params:

        * `repo_name`: A string specifying the repo name.
        * `reverse`: Optional. If true, returns branches oldest to newest.
        """
        return self._req(
            Service.PFS,
            "ListBranch",
            repo=pfs_proto.Repo(name=repo_name, type="user"),
            reverse=reverse,
        )

    def delete_branch(self, repo_name, branch_name, force=None):
        """
        Deletes a branch, but leaves the commits themselves intact. In other
        words, those commits can still be accessed via commit IDs and other
        branches they happen to be on.

        Params:

        * `repo_name`: A string specifying the repo name.
        * `branch_name`: A string specifying the name of the branch to delete.
        * `force`: A bool specifying whether to force the branch deletion.
        """
        self._req(
            Service.PFS,
            "DeleteBranch",
            branch=pfs_proto.Branch(
                repo=pfs_proto.Repo(name=repo_name, type="user"), name=branch_name
            ),
            force=force,
        )

    @contextmanager
    def modify_file_client(self, commit):
        """
        A context manager that gives a `ModifyFileClient`. When the context
        manager exits, any operations enqueued from the `ModifyFileClient` are
        executed in a single, atomic `ModifyFile` call.
        """
        pfc = ModifyFileClient(commit)
        yield pfc
        self._req(Service.PFS, "ModifyFile", req=pfc._reqs())

    def put_file_bytes(
        self,
        commit,
        path,
        value,
        datum=None,
        append=None,
    ):
        """
        Uploads a PFS file from a file-like object, bytestring, or iterator
        of bytestrings.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: A string specifying the path in the repo the file(s) will be
        written to.
        * `value`: The file contents as bytes, represented as a file-like
        object, bytestring, or iterator of bytestrings.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """
        with self.modify_file_client(commit) as pfc:
            if hasattr(value, "read"):
                pfc.put_file_from_fileobj(
                    path,
                    value,
                    datum=datum,
                    append=append,
                )
            else:
                pfc.put_file_from_bytes(
                    path,
                    value,
                    datum=datum,
                    append=append,
                )

    def put_file_url(
        self,
        commit,
        path,
        url,
        recursive=None,
        datum=None,
        append=None,
    ):
        """
        Puts a file using the content found at a URL. The URL is sent to the
        server which performs the request.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: A string specifying the path to the file.
        * `url`: A string specifying the url of the file to put.
        * `recursive`: allow for recursive scraping of some types URLs, for
        example on s3:// URLs.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """

        with self.modify_file_client(commit) as pfc:
            pfc.put_file_from_url(
                path,
                url,
                recursive=recursive,
                datum=datum,
                append=append,
            )

    def copy_file(
        self,
        source_commit,
        source_path,
        dest_commit,
        dest_path,
        datum=None,
        append=None,
    ):
        """
        Efficiently copies files already in PFS. Note that the destination
        repo cannot be an output repo, or the copy operation will (as of
        1.9.0) silently fail.

        Params:

        * `source_commit`: A tuple, string, or `Commit` object representing the
        commit for the source file.
        * `source_path`: A string specifying the path of the source file.
        * `dest_commit`: A tuple, string, or `Commit` object representing the
        commit for the destination file.
        * `dest_path`: A string specifying the path of the destination file.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """
        with self.modify_file_client(dest_commit) as pfc:
            pfc.copy_file(
                source_commit, source_path, dest_path, datum=datum, append=append
            )

    def get_file(self, commit, path, datum=None, URL=None, offset=None):
        """
        Returns a `PFSFile` object, containing the contents of a file stored
        in PFS.

        Params:

        * `commit`: A tuple, dict, or `Commit` object representing the
        commit.
        * `path`: A string specifying the path of the file.
        * `datum`: A string to distinguish files by.
        * `URL`: A string that specifies an object storage URL that the file
        will be uploaded to.
        * `offset`: An int that allows file read to begin at `offset` number of
        bytes.
        """
        res = self._req(
            Service.PFS,
            "GetFile",
            file=pfs_proto.File(commit=commit_from(commit), path=path, datum=datum),
            URL=URL,
            offset=offset,
        )
        return PFSFile(io.BytesIO(next(res).value))

    def get_file_tar(self, commit, path, datum=None, URL=None, offset=None):
        """
        Returns a `PFSFile` object, containing the contents of a file stored
        in PFS.

        Params:

        * `commit`: A tuple, dict, or `Commit` object representing the
        commit.
        * `path`: A string specifying the path of the file.
        * `datum`: A string to distinguish files by.
        * `URL`: A string that specifies an object storage URL that the file
        will be uploaded to.
        * `offset`: An int that allows file read to begin at `offset` number of
        bytes.
        """
        res = self._req(
            Service.PFS,
            "GetFileTAR",
            req=pfs_proto.GetFileRequest(
                file=pfs_proto.File(commit=commit_from(commit), path=path, datum=datum),
                URL=URL,
                offset=offset,
            ),
        )
        return PFSFile(io.BytesIO(next(res).value), is_tar=True)

    def inspect_file(self, commit, path):
        """
        Inspects a file. Returns a `FileInfo` object.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: A string specifying the path to the file.
        """
        return self._req(
            Service.PFS,
            "InspectFile",
            file=pfs_proto.File(commit=commit_from(commit), path=path),
        )

    def list_file(self, commit, path):
        """
        Lists the files in a directory.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: The path to the directory.
        """
        return self._req(
            Service.PFS,
            "ListFile",
            file=pfs_proto.File(commit=commit_from(commit), path=path),
            # history=history,
        )

    def walk_file(self, commit, path):
        """
        Walks over all descendant files in a directory. Returns a generator of
        `FileInfo` objects.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: The path to the directory.
        """
        return self._req(
            Service.PFS,
            "WalkFile",
            file=pfs_proto.File(commit=commit_from(commit), path=path),
        )

    def glob_file(self, commit, pattern):
        """
        Lists files that match a glob pattern. Yields `FileInfo` objects.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `pattern`: A string representing a glob pattern.
        """
        return self._req(
            Service.PFS, "GlobFile", commit=commit_from(commit), pattern=pattern
        )

    def delete_file(self, commit, path):
        """
        Deletes a file from a Commit. DeleteFile leaves a tombstone in the
        Commit, assuming the file isn't written to later attempting to get the
        file from the finished commit will result in not found error. The file
        will of course remain intact in the Commit's parent.

        Params:

        * `commit`: A tuple, string, or `Commit` object representing the
        commit.
        * `path`: The path to the file.
        """
        with self.modify_file_client(commit) as pfc:
            pfc.delete_file(path)

    def fsck(self, fix=None):
        """
        Performs a file system consistency check for PFS.
        """
        return self._req(Service.PFS, "Fsck", fix=fix)

    def diff_file(
        self, new_commit, new_path, old_commit=None, old_path=None, shallow=None
    ):
        """
        Diffs two files. If `old_commit` or `old_path` are not specified, the
        same path in the parent of the file specified by `new_commit` and
        `new_path` will be used.

        Params:

        * `new_commit`: A tuple, string, or `Commit` object representing the
        commit for the new file.
        * `new_path`: A string specifying the path of the new file.
        * `old_commit`: A tuple, string, or `Commit` object representing the
        commit for the old file.
        * `old_path`: A string specifying the path of the old file.
        * `shallow`: An optional bool specifying whether to do a shallow diff.
        """

        if old_commit is not None and old_path is not None:
            old_file = pfs_proto.File(commit=commit_from(old_commit), path=old_path)
        else:
            old_file = None

        return self._req(
            Service.PFS,
            "DiffFile",
            new_file=pfs_proto.File(commit=commit_from(new_commit), path=new_path),
            old_file=old_file,
            shallow=shallow,
        )


class ModifyFileClient:
    """
    `ModifyFileClient` puts or deletes PFS files atomically.
    """

    def __init__(self, commit):
        self._ops = []
        self.commit = commit_from(commit)

    def _reqs(self):
        yield pfs_proto.ModifyFileRequest(set_commit=self.commit)
        for op in self._ops:
            yield from op.reqs()

    def put_file_from_filepath(
        self,
        pfs_path,
        local_path,
        datum=None,
        append=None,
    ):
        """
        Uploads a PFS file from a local path at a specified path. This will
        lazily open files, which will prevent too many files from being
        opened, or too much memory being consumed, when atomically putting
        many files.

        Params:

        * `pfs_path`: A string specifying the path in the repo the file(s)
        will be written to.
        * `local_path`: A string specifying the local file path.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        * `datum`: A string for the file datum.
        """
        self._ops.append(
            AtomicModifyFilepathOp(
                pfs_path,
                local_path,
                datum,
                append,
            )
        )

    def put_file_from_fileobj(
        self,
        path,
        value,
        datum=None,
        append=None,
    ):
        """
        Uploads a PFS file from a file-like object.

        Params:

        * `path`: A string specifying the path in the repo the file(s) will be
        written to.
        * `value`: The file-like object.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """
        self._ops.append(
            AtomicModifyFileobjOp(
                path,
                value,
                datum,
                append,
            )
        )

    def put_file_from_bytes(
        self,
        path,
        value,
        datum=None,
        append=None,
    ):
        """
        Uploads a PFS file from a bytestring.

        Params:

        * `path`: A string specifying the path in the repo the file(s) will be
        written to.
        * `value`: The file contents as a bytestring.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """
        self.put_file_from_fileobj(
            path,
            io.BytesIO(value),
            datum=datum,
            append=append,
        )

    def put_file_from_url(
        self,
        path,
        url,
        datum=None,
        append=None,
        recursive=None,
    ):
        """
        Puts a file using the content found at a URL. The URL is sent to the
        server which performs the request.

        Params:

        * `path`: A string specifying the path to the file.
        * `url`: A string specifying the url of the file to put.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        * `recursive`: allow for recursive scraping of some types URLs, for
        example on s3:// URLs.
        """
        self._ops.append(
            AtomicModifyFileURLOp(
                path,
                url,
                datum=datum,
                append=append,
                recursive=recursive,
            )
        )

    def delete_file(self, path, datum=None):
        """
        Deletes a file.

        Params:

        * `path`: The path to the file.
        * `datum`: A string for the file datum.
        """
        self._ops.append(AtomicDeleteFileOp(path, datum=datum))

    def copy_file(
        self, source_commit, source_path, dest_path, datum=None, append=False
    ):
        """
        Copy a file.

        Params:

        * `source_commit`: The commit the source file is in.
        * `source_path`: The path to the source file.
        * `dest_path`: The path to the destination file.
        * `datum`: A string for the file datum.
        * `append`: An optional bool, if true the data is appended to the file,
        if it already exists.
        """
        self._ops.append(
            AtomicCopyFileOp(
                source_commit,
                source_path,
                dest_path,
                datum=datum,
                append=append,
            )
        )


class AtomicOp:
    """
    Represents an operation in a `ModifyFile` call.
    """

    def __init__(self, path, datum):
        self.path = path
        self.datum = datum

    def reqs(self):
        """
        Yields one or more protobuf `ModifyFileRequests`, which are then
        enqueued into the request's channel.
        """
        pass


class AtomicModifyFilepathOp(AtomicOp):
    """
    A `ModifyFile` operation to put a file locally stored at a given path. This
    file is opened on-demand, which helps with minimizing the number of open
    files.
    """

    def __init__(self, pfs_path, local_path, datum=None, append=False):
        super().__init__(pfs_path, datum)
        self.local_path = local_path
        self.append = append

    def reqs(self):
        if not self.append:
            yield delete_file_req(self.path, self.datum)
        with open(self.local_path, "rb") as f:
            yield add_file_req(path=self.path, datum=self.datum)
            for i, chunk in enumerate(f):
                yield add_file_req(path=self.path, datum=self.datum, chunk=chunk)


class AtomicModifyFileobjOp(AtomicOp):
    """A `ModifyFile` operation to put a file from a file-like object."""

    def __init__(self, path, fobj, datum=None, append=False):
        super().__init__(path, datum)
        self.fobj = fobj
        self.append = append

    def reqs(self):
        if not self.append:
            yield delete_file_req(self.path, self.datum)
        yield add_file_req(path=self.path, datum=self.datum)
        for i in itertools.count():
            chunk = self.fobj.read(BUFFER_SIZE)
            if len(chunk) == 0:
                return
            yield add_file_req(path=self.path, datum=self.datum, chunk=chunk)


class AtomicModifyFileURLOp(AtomicOp):
    """A `ModifyFile` operation to put a file from a URL."""

    def __init__(self, path, url, datum=None, append=False, recursive=False):
        super().__init__(path, datum)
        self.url = url
        self.recursive = recursive
        self.append = append

    def reqs(self):
        if not self.append:
            yield delete_file_req(self.path, self.datum)
        yield pfs_proto.ModifyFileRequest(
            add_file=pfs_proto.AddFile(
                path=self.path,
                datum=self.datum,
                url=pfs_proto.AddFile.URLSource(
                    URL=self.url,
                    recursive=self.recursive,
                ),
            ),
        )


class AtomicCopyFileOp(AtomicOp):
    """A `ModifyFile` operation to copy a file."""

    def __init__(self, source_commit, source_path, dest_path, datum=None, append=False):
        super().__init__(dest_path, datum)
        self.source_commit = commit_from(source_commit)
        self.source_path = source_path
        self.dest_path = dest_path
        self.append = append

    def reqs(self):
        yield pfs_proto.ModifyFileRequest(
            copy_file=pfs_proto.CopyFile(
                append=self.append,
                datum=self.datum,
                dst=self.dest_path,
                src=pfs_proto.File(commit=self.source_commit, path=self.source_path),
            ),
        )


class AtomicDeleteFileOp(AtomicOp):
    """A `ModifyFile` operation to delete a file."""

    def __init__(self, pfs_path, datum=None):
        super().__init__(pfs_path, datum)

    def reqs(self):
        yield delete_file_req(self.path, self.datum)


def add_file_req(path=None, datum=None, chunk=None):
    return pfs_proto.ModifyFileRequest(
        add_file=pfs_proto.AddFile(
            path=path, datum=datum, raw=wrappers_pb2.BytesValue(value=chunk)
        ),
    )


def delete_file_req(path=None, datum=None):
    return pfs_proto.ModifyFileRequest(
        delete_file=pfs_proto.DeleteFile(path=path, datum=datum)
    )
